clear; clc; % close all;

% ----- Grids ----- %
L = 5e-2;
Nxy = 512;
dxy = L/Nxy;
[xs,ys] = meshgrid((-Nxy/2:Nxy/2-1)*dxy);
[~,rhos] = cart2pol(xs,ys);
df = 1/(Nxy*dxy);
[fx,fy] = meshgrid((-Nxy/2:Nxy/2-1)*df);
[~,rhof] = cart2pol(fx,fy);
% ----------------- %

% ----- Source ----- %
sigma = L/8;
delU = L/30;
sigmaI = 1;
meanI = 1;
%%% Shape %%%
tau = exp(-rhos.^2/(4*sigma^2));
%%% Phase %%%
sigma_s = 2*pi;
pS = sqrt(2*pi)*sigma_s^2*delU*exp(-pi^2*sigma_s^2*delU^2*rhof.^2);
%%% Amplitude %%%
pI = meanI*sigmaI*sqrt(pi)*delU*exp(-pi^2/2*delU^2*rhof.^2);
% ------------------ %

% ----- Theory ----- %
[x1,x2] = ndgrid((-Nxy/2:Nxy/2-1)*dxy);
Wx1x2_thy = exp(-x1.^2/(4*sigma^2)).*exp(-x2.^2/(4*sigma^2)) ...
    .*exp(-(x1-x2).^2/(2*delU^2));
BI_thy = abs(exp(-(x1-x2).^2/(2*delU^2))).^2;
% ------------------ %

% ----- Simulation ----- %
numtrials = 10e3;
Wx1x2 = zeros(Nxy);
Ix1x2 = zeros(Nxy);
I = zeros(Nxy);
Uavg = zeros(Nxy);
iIx1x2 = zeros(Nxy);
iI_binedges = linspace(0,10,501);
iI_counts = zeros(1,length(iI_binedges)-1);
U_binedges = linspace(-3,3,501);
Ur_counts = zeros(1,length(U_binedges)-1);
Ui_counts = zeros(1,length(U_binedges)-1);

Wx1x2_ver = zeros(Nxy);
Ix1x2_ver = zeros(Nxy);
I_ver = zeros(Nxy);
Uavg_ver = zeros(Nxy);
iIx1x2_ver = zeros(Nxy);
iI_counts_ver = zeros(1,length(iI_binedges)-1);
Ur_counts_ver = zeros(1,length(U_binedges)-1);
Ui_counts_ver = zeros(1,length(U_binedges)-1);
for nn = 1:numtrials
    
   disp(['Trial ' num2str(nn) ' of ' num2str(numtrials)]);
   
   % ----- Verification ----- %
   %%% Complex Screen Field %%%
   r = randn(Nxy) + 1j*randn(Nxy);
   T = fftshift(fft2(fftshift(r.*sqrt(2*pi*delU^2*exp(-2*pi^2*delU^2*rhof.^2)) ...
       *1/sqrt(2)*sqrt(df)*sqrt(df))));
   U_ver = tau.*T;      
   % ------------------------ %
   
   % ----- New ----- %
   %%% Phase Instance %%%
   rS = randn(Nxy) + 1j*randn(Nxy);
   S = real(fftshift(fft2(fftshift(rS.*pS*sqrt(df)*sqrt(df)))));   
   %%% Amplitude Instance %%%
   ri = randn(Nxy) + 1j*randn(Nxy);
   iI = real(fftshift(fft2(fftshift(ri.*pI*sqrt(df)*sqrt(df)))));
   iI = reshape(expinv(normcdf(iI(:),0,sigmaI),meanI),[Nxy Nxy]);   
   %%% Field Instance %%%
   U = tau.*sqrt(iI).*exp(1j*S);
   % --------------- %
   
   %%% Moments %%%
   Uavg_ver = Uavg_ver + U_ver;
   Ur_counts_ver = Ur_counts_ver + histcounts(real(T(:)),U_binedges);
   Ui_counts_ver = Ui_counts_ver + histcounts(imag(T(:)),U_binedges);   
   iI_counts_ver = iI_counts_ver + histcounts(abs(T(:)).^2,iI_binedges);    
   I_ver = I_ver + abs(U_ver).^2;
   Wx1x2_ver = Wx1x2_ver + transpose(U_ver(Nxy/2+1,:))*conj(U_ver(Nxy/2+1,:));
   Ix1x2_ver = Ix1x2_ver + transpose(abs(U_ver(Nxy/2+1,:)).^2)*conj(abs(U_ver(Nxy/2+1,:)).^2);
   
   Uavg = Uavg + U;
   Ur_counts = Ur_counts + histcounts(real(sqrt(iI(:)).*exp(1j*S(:))),U_binedges);
   Ui_counts = Ui_counts + histcounts(imag(sqrt(iI(:)).*exp(1j*S(:))),U_binedges);   
   iI_counts = iI_counts + histcounts(iI(:),iI_binedges);    
   I = I + abs(U).^2;
   Wx1x2 = Wx1x2 + transpose(U(Nxy/2+1,:))*conj(U(Nxy/2+1,:));
   Ix1x2 = Ix1x2 + transpose(abs(U(Nxy/2+1,:)).^2)*conj(abs(U(Nxy/2+1,:)).^2);
   
end
Uavg_ver = Uavg_ver/numtrials;
I_ver = I_ver/numtrials;
Wx1x2_ver = Wx1x2_ver/numtrials;
Ix1x2_ver = Ix1x2_ver/numtrials;
BI_ver = (Ix1x2_ver - transpose(I_ver(Nxy/2+1,:))*I_ver(Nxy/2+1,:)) ...
    ./(transpose(I_ver(Nxy/2+1,:))*I_ver(Nxy/2+1,:));

Uavg = Uavg/numtrials;
I = I/numtrials;
Wx1x2 = Wx1x2/numtrials;
Ix1x2 = Ix1x2/numtrials;
BI = (Ix1x2 - transpose(I(Nxy/2+1,:))*I(Nxy/2+1,:)) ...
    ./(transpose(I(Nxy/2+1,:))*I(Nxy/2+1,:));
% ---------------------- %

save('Gaussian_Schell_Sim_Results.mat','-v7.3');

figure;
subplot(241);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(242);
imagesc(xs(1,:)*100,ys(:,1)*100,angle(U_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(243);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U_ver).^2); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(245);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(246);
imagesc(xs(1,:)*100,ys(:,1)*100,angle(U)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(247);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U).^2); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(2,4,[4 8]);
bin_c = diff(iI_binedges)/2 + iI_binedges(1:end-1);
semilogy(log(bin_c),exppdf(bin_c,1),'b');
hold on;
h = histogram('BinEdges',iI_binedges,'BinCounts',iI_counts_ver,'normalization','pdf','visible','off');
semilogy(log(bin_c),h.Values,'r');
h = histogram('BinEdges',iI_binedges,'BinCounts',iI_counts,'normalization','pdf','visible','off');
semilogy(log(bin_c),h.Values,'g');
set(gca,'ticklabelinterpreter','latex');
ylabel('$p\left(I\right)$','interpreter','latex');
xlabel('$\mbox{ln}\left(I\right)$','interpreter','latex');
axis tight; grid on;
ylim([1e-5 2]);

figure;
subplot(241);
imagesc(xs(1,:)*100,ys(:,1)*100,real(Wx1x2_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(242);
imagesc(xs(1,:)*100,ys(:,1)*100,real(Wx1x2_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(243);
imagesc(xs(1,:)*100,ys(:,1)*100,real(Wx1x2)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(244);
plot(xs(1,:)*100,diag(real(Wx1x2_thy))); 
hold on;
plot(xs(1,:)*100,diag(real(Wx1x2_ver))); 
plot(xs(1,:)*100,diag(real(Wx1x2))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;
subplot(245);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(Wx1x2_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(246);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(Wx1x2_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(247);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(Wx1x2)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(248);
plot(xs(1,:)*100,diag(imag(Wx1x2_thy))); 
hold on;
plot(xs(1,:)*100,diag(imag(Wx1x2_ver))); 
plot(xs(1,:)*100,diag(imag(Wx1x2))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;

figure;
subplot(241);
imagesc(xs(1,:)*100,ys(:,1)*100,real(BI_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(242);
imagesc(xs(1,:)*100,ys(:,1)*100,real(BI_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(243);
imagesc(xs(1,:)*100,ys(:,1)*100,real(BI)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(244);
plot(xs(1,:)*100,diag(real(BI_thy))); 
hold on;
plot(xs(1,:)*100,diag(real(BI_ver))); 
plot(xs(1,:)*100,diag(real(BI))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;
subplot(245);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(BI_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(246);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(BI_ver)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(247);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(BI)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(248);
plot(xs(1,:)*100,diag(imag(BI_thy))); 
hold on;
plot(xs(1,:)*100,diag(imag(BI_ver))); 
plot(xs(1,:)*100,diag(imag(BI))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;