clear; clc; % close all;

% ----- Grids ----- %
L = 5e-2;
Nxy = 512;
dxy = L/Nxy;
[xs,ys] = meshgrid((-Nxy/2:Nxy/2-1)*dxy);
[~,rhos] = cart2pol(xs,ys);
% ----------------- %

% ----- Source ----- %
sigma = L/8;
gam = [-L/12 -L/10];
delU = L/10;
delI = delU/5;
sigmaI = 0.5;
meanI = 1;
%%% Shape %%%
tau = exp(-rhos.^2/(4*sigma^2));
%%% Phase %%%
Nvx = 2*Nxy;
sigma_s = 2*pi;
maxVx = sqrt(-8*log(0.01)/(sigma_s^2*delU^4));
dvx = 2*maxVx/Nvx; 
vx = (-Nvx/2:Nvx/2-1)*dvx;
rhoHS = linspace(0,sqrt(2)*L/2 + norm(gam),Nvx);
HS = exp(-1j*repmat(rhoHS(:).^2,[1 length(vx)]) ...
    .*repmat(vx,[length(rhoHS(:)) 1]));
% HS = exp(-1j*repmat((xs(:)-gam(1)).^2+(ys(:)-gam(2)).^2,[1 length(vx)]) ...
%     .*repmat(vx,[length(xs(:)) 1]));
pS = sqrt(sigma_s^3*delU^2/(2*sqrt(pi)))*exp(-sigma_s^2*delU^4/8*vx.^2);
%%% Amplitude %%%
rhoHI = linspace(0,sqrt(2)*L/2,Nvx);
HI = exp(-1j*repmat(rhoHI(:),[1 length(vx)]).*repmat(vx,[length(rhoHI(:)) 1]));
% HI = exp(-1j*repmat(rhos(:),[1 length(vx)]).*repmat(vx,[length(xs(:)) 1]));
pI = meanI*sigmaI*sqrt(delI/2)*rect(delI/2*vx,1);
% ------------------ %

% ----- Theory ----- %
[x1,x2] = ndgrid((-Nxy/2:Nxy/2-1)*dxy);
Wx1x2_thy = meanI*exp(-x1.^2/(4*sigma^2)).*exp(-x2.^2/(4*sigma^2)) ...
    .*exp(-((x1-gam(1)).^2-(x2-gam(1)).^2).^2/delU^4);
BI_thy = sigmaI^2*sinc(1/pi*(abs(x1)-abs(x2))/delI);
Ix1x2_thy = transpose(meanI*abs(tau(Nxy/2+1,:)).^2)*meanI*abs(tau(Nxy/2+1,:)).^2.*(BI_thy + 1);
% ------------------ %

% ----- Simulation ----- %
numtrials = 10e3;
Wx1x2 = zeros(Nxy);
Ix1x2 = zeros(Nxy);
I = zeros(Nxy);
Uavg = zeros(Nxy);
iIx1x2 = zeros(Nxy);
iI_binedges = linspace(0,10,501);
iI_counts = zeros(1,length(iI_binedges)-1);
U_binedges = linspace(-3,3,501);
Ur_counts = zeros(1,length(U_binedges)-1);
Ui_counts = zeros(1,length(U_binedges)-1);
for nn = 1:numtrials
    
   disp(['Trial ' num2str(nn) ' of ' num2str(numtrials)]);
   
   %%% Phase Instance %%%
   rS = randn(size(vx)) + 1j*randn(size(vx));
   S = real(HS*transpose(pS.*rS)*sqrt(dvx));
%    S = reshape(S,[Nxy Nxy]);
   S = interp1(rhoHS,S,sqrt((xs-gam(1)).^2+(ys-gam(2)).^2),'spline',0);
   %%% Amplitude Instance %%%
   ri = randn(size(vx)) + 1j*randn(size(vx));
   iI = real(HI*transpose(pI.*ri)*sqrt(dvx));
   iI = interp1(rhoHI,iI,rhos,'spline',0);
   iI = reshape(logninv(normcdf(iI(:),0,sigmaI), ...
       log(meanI/sqrt(sigmaI^2+1)),sqrt(log(sigmaI^2+1))),[Nxy Nxy]);
   
   %%% Field Instance %%%
   U = tau.*sqrt(iI).*exp(1j*S);
   
   %%% Moments %%%
   Uavg = Uavg + U;
   Ur_counts = Ur_counts + histcounts(real(sqrt(iI(:)).*exp(1j*S(:))),U_binedges);
   Ui_counts = Ui_counts + histcounts(imag(sqrt(iI(:)).*exp(1j*S(:))),U_binedges);
   iI_counts = iI_counts + histcounts(iI(:),iI_binedges);    
   I = I + abs(U).^2;
   Wx1x2 = Wx1x2 + transpose(U(Nxy/2+1,:))*conj(U(Nxy/2+1,:));
   Ix1x2 = Ix1x2 + transpose(abs(U(Nxy/2+1,:)).^2)*conj(abs(U(Nxy/2+1,:)).^2);
   
   %%% Correlation Coefficient %%%
   Wx1x2_corrcoeff(nn) = (Wx1x2(:)/nn-mean2(Wx1x2/nn))'*(Wx1x2_thy(:)-mean2(Wx1x2_thy)) ...
       ./(norm(Wx1x2/nn-mean2(Wx1x2/nn),'fro')*norm(Wx1x2_thy-mean2(Wx1x2_thy),'fro'));
   BI_corrcoeff(nn) = corr2((Ix1x2/nn - transpose(I(Nxy/2+1,:)/nn)*I(Nxy/2+1,:)/nn) ...
       ./(transpose(I(Nxy/2+1,:)/nn)*I(Nxy/2+1,:)/nn),BI_thy);
    
end
Uavg = Uavg/numtrials;
I = I/numtrials;
Wx1x2 = Wx1x2/numtrials;
Ix1x2 = Ix1x2/numtrials;
BI = (Ix1x2 - transpose(I(Nxy/2+1,:))*I(Nxy/2+1,:))./(transpose(I(Nxy/2+1,:))*I(Nxy/2+1,:));
% ---------------------- %

save('NUC_Phase_Pseudo_Schell_Amp_Sim_Results.mat','-v7.3');

figure;
subplot(221);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(222);
imagesc(xs(1,:)*100,ys(:,1)*100,angle(U)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(223);
imagesc(xs(1,:)*100,ys(:,1)*100,abs(U).^2); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(224);
bin_c = diff(iI_binedges)/2 + iI_binedges(1:end-1);
semilogy(log(bin_c),lognpdf(bin_c,log(meanI/sqrt(sigmaI^2+1)),sqrt(log(sigmaI^2+1))));
hold on;
h = histogram('BinEdges',iI_binedges,'BinCounts',iI_counts,'normalization','pdf','visible','off');
semilogy(log(bin_c),h.Values);
set(gca,'ticklabelinterpreter','latex');
ylabel('$p\left(I\right)$','interpreter','latex');
xlabel('$\mbox{ln}\left(I\right)$','interpreter','latex');
axis tight; grid on;

figure;
subplot(231);
imagesc(xs(1,:)*100,ys(:,1)*100,real(Wx1x2_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(232);
imagesc(xs(1,:)*100,ys(:,1)*100,real(Wx1x2)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(233);
plot(xs(1,:)*100,diag(real(Wx1x2_thy))); 
hold on;
plot(xs(1,:)*100,diag(real(Wx1x2))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;
subplot(234);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(Wx1x2_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(235);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(Wx1x2)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(236);
plot(xs(1,:)*100,diag(imag(Wx1x2_thy))); 
hold on;
plot(xs(1,:)*100,diag(imag(Wx1x2))); 
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
axis tight; grid on;

% figure;
% subplot(231);
% imagesc(xs(1,:)*100,ys(:,1)*100,real(Ix1x2_thy)); axis image; colorbar;
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% subplot(232);
% imagesc(xs(1,:)*100,ys(:,1)*100,real(Ix1x2)); axis image; colorbar;
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% subplot(233);
% plot(xs(1,:)*100,diag(real(Ix1x2_thy))); 
% hold on;
% plot(xs(1,:)*100,diag(real(Ix1x2))); 
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% axis tight; grid on;
% subplot(234);
% imagesc(xs(1,:)*100,ys(:,1)*100,imag(Ix1x2_thy)); axis image; colorbar;
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% subplot(235);
% imagesc(xs(1,:)*100,ys(:,1)*100,imag(Ix1x2)); axis image; colorbar;
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% subplot(236);
% plot(xs(1,:)*100,diag(imag(Ix1x2_thy))); 
% hold on;
% plot(xs(1,:)*100,diag(imag(Ix1x2))); 
% xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
% set(gca,'ticklabelinterpreter','latex');
% axis tight; grid on;

figure;
subplot(221);
imagesc(xs(1,:)*100,ys(:,1)*100,real(BI_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(222);
imagesc(xs(1,:)*100,ys(:,1)*100,real(BI)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(223);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(BI_thy)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');
subplot(224);
imagesc(xs(1,:)*100,ys(:,1)*100,imag(BI)); axis image; colorbar;
xlabel('$x_2 \mbox{ (cm)}$','interpreter','latex');
ylabel('$x_1 \mbox{ (cm)}$','interpreter','latex');
set(gca,'ticklabelinterpreter','latex');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = rect(x,D)
%
%
%
%
%

y = zeros(size(x));
y(abs(x) < D/2) = 1;
y(abs(x) == D/2) = 1/2;
end

